@@ -1,6 +1,6 @@
-/* 1-2.cpp:
+/* 1-3.cpp:
 
-   Copyright (C) 1996-2017 Christian Schenk
+   Copyright (C) 2018 Christian Schenk
 
    This file is part of the MiKTeX Core Library.
 
@@ -28,14 +28,11 @@
 using namespace MiKTeX::Test;
 using namespace std;
 
-BEGIN_TEST_SCRIPT("process-1-2");
+BEGIN_TEST_SCRIPT("process-1-3");
 
 BEGIN_TEST_FUNCTION(1);
 {
-  for (const std::string& arg : vecArgs)
-  {
-    TEST(printf("%s\n", arg.c_str()) == strlen(arg.c_str()) + 1);
-  }
+  throw TestException("process-1-3", "{a}errorMessage{b}", "{a}description{b}", "{a}remedy{b}", "tag", { "a", "x", "b", "y" }, MIKTEX_SOURCE_LOCATION());
 }
 END_TEST_FUNCTION();
 
