@@ -2,7 +2,7 @@
 ** CMapReader.cpp                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,9 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#if defined(MIKTEX)
+#include <config.h>
+#endif
 #include <algorithm>
 #include <array>
 #include <fstream>
@@ -42,7 +45,7 @@ using namespace std;
 unique_ptr<CMap> CMapReader::read (const string &fname) {
 	if (const char *path = FileFinder::instance().lookup(fname, "cmap", false)) {
 #if defined(MIKTEX_WINDOWS)
-                ifstream ifs(EXPATH_(path));
+		ifstream ifs(EXPATH_(path));
 #else
 		ifstream ifs(path);
 #endif
