@@ -2,7 +2,7 @@
 ** FontManager.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2023 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,16 +18,22 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#if defined(MIKTEX)
+#include <config.h>
+#endif
 #include <cstring>
 #include <cstdlib>
 #include <fstream>
 #include <set>
 #include "CMap.hpp"
 #include "Font.hpp"
+#include "fonts/Base14Fonts.hpp"
+#include "FontEngine.hpp"
 #include "FontManager.hpp"
 #include "FileFinder.hpp"
 #include "FileSystem.hpp"
 #include "Message.hpp"
+#include "SVGTree.hpp"
 
 using namespace std;
 
@@ -80,6 +86,18 @@ int FontManager::fontID (const string &name) const {
 }
 
 
+int FontManager::fontID (string name, double ptsize) const {
+	std::replace(name.begin(), name.end(), '+', '-');
+	for (auto it = _fonts.begin(); it != _fonts.end(); ++it) {
+		if (auto nativeFont = font_cast<NativeFont*>(it->get())) {
+			if (nativeFont->name() == name && nativeFont->scaledSize() == ptsize)
+				return int(std::distance(_fonts.begin(), it));
+		}
+	}
+	return -1;
+}
+
+
 int FontManager::fontnum (int id) const {
 	if (id < 0 || size_t(id) > _fonts.size())
 		return -1;
@@ -129,6 +147,14 @@ Font* FontManager::getFont (const string &name) const {
 }
 
 
+Font* FontManager::getFont (const string &name, double ptsize) {
+	int id = fontID(name, ptsize);
+	if (id < 0)
+		return nullptr;
+	return _fonts[id].get();
+}
+
+
 Font* FontManager::getFontById (int id) const {
 	if (id < 0 || size_t(id) >= _fonts.size())
 		return nullptr;
@@ -145,7 +171,7 @@ const VirtualFont* FontManager::getVF () const {
 static unique_ptr<Font> create_font (const string &filename, const string &fontname, int fontindex, uint32_t checksum, double dsize, double ssize) {
 	string ext;
 	if (const char *dot = strrchr(filename.c_str(), '.'))
-		ext = dot+1;
+		ext = util::tolower(dot+1);
 	if (!ext.empty() && FileFinder::instance().lookup(filename)) {
 		if (ext == "pfb")
 			return PhysicalFont::create(fontname, checksum, dsize, ssize, PhysicalFont::Type::PFB);
@@ -219,18 +245,16 @@ int FontManager::registerFont (uint32_t fontnum, const string &name, uint32_t ch
 				missing_fonts.insert(filename);
 			}
 		}
-		_name2id[name] = newid;
+		_name2id.emplace(name, newid);
 	}
-	_fonts.emplace_back(std::move(newfont));
+	_fonts.push_back(std::move(newfont));
 	if (_vfStack.empty())  // register font referenced in dvi file?
 		_num2id[fontnum] = newid;
 	else {  // register font referenced in vf file
 		const VirtualFont *vf = _vfStack.top();
 		_vfnum2id[vf][fontnum] = newid;
-		if (_vfFirstFontNumMap.find(vf) == _vfFirstFontNumMap.end()) { // first fontdef of VF?
-			_vfFirstFontNumMap.emplace(vf, fontnum);
-			_vfFirstFontMap.emplace(vf, _fonts.back().get());
-		}
+		_vfFirstFontNumMap.emplace(vf, fontnum);
+		_vfFirstFontMap.emplace(vf, _fonts.back().get());
 	}
 	return newid;
 }
@@ -258,7 +282,7 @@ int FontManager::registerFont (uint32_t fontnum, const string &filename, double
  *  @param[in] style font style parameters
  *  @param[in] color global font color
  *  @return global font id */
-int FontManager::registerFont (uint32_t fontnum, string filename, int fontIndex, double ptsize, const FontStyle &style, Color color) {
+int FontManager::registerFont (uint32_t fontnum, const string &filename, int fontIndex, double ptsize, const FontStyle &style, Color color) {
 	int id = fontID(fontnum);
 	if (id >= 0)
 		return id;
@@ -269,12 +293,17 @@ int FontManager::registerFont (uint32_t fontnum, string filename, int fontIndex,
 	const int newid = _fonts.size();   // the new font gets this ID
 	auto it = _name2id.find(fontname);
 	if (it != _name2id.end()) {  // font with same name already registered?
-		if (auto font = dynamic_cast<NativeFont*>(_fonts[it->second].get()))
+		if (auto font = font_cast<NativeFont*>(_fonts[it->second].get()))
 			newfont = font->clone(ptsize, style, color);
 	}
 	else {
-		if (!FileSystem::exists(path))
-			path = FileFinder::instance().lookup(filename, false);
+		if (!FileSystem::exists(path)) {
+			const char *fontFormats[] = {nullptr, "otf", "ttf"};
+			for (const char *format : fontFormats) {
+				if ((path = FileFinder::instance().lookup(filename, format, false)) != nullptr)
+					break;
+			}
+		}
 		if (path) {
 			newfont.reset(new NativeFontImpl(path, fontIndex, ptsize, style, color));
 			newfont->findAndAssignBaseFontMap();
@@ -289,14 +318,48 @@ int FontManager::registerFont (uint32_t fontnum, string filename, int fontIndex,
 				missing_fonts.insert(filename);
 			}
 		}
-		_name2id[fontname] = newid;
+		_name2id.emplace(fontname, newid);
 	}
-	_fonts.emplace_back(std::move(newfont));
+	_fonts.push_back(std::move(newfont));
 	_num2id[fontnum] = newid;
 	return newid;
 }
 
 
+/** Registers a native font that is referenced by its name instead of a DVI font number.
+ *  @param[in] fname filename/path of the font file
+ *  @param[in] fontname font name used if the font file doesn't provide one
+ *  @param[in] ptsize font size in PS points
+ *  return global ID assigned to the font */
+int FontManager::registerFont (const string &fname, string fontname, double ptsize) {
+	if (fname.empty())
+		return -1;
+	if (fname.size() > 6 && fname.substr(0,6) == "sys://") {
+		fontname = fname.substr(6);
+		if (!find_base14_font(fontname))
+			return -1;
+	}
+	else if (!FileSystem::exists(fname) || (fontname.empty() && (fontname = FontEngine::instance().getPSName(fname)).empty()))
+		return -1;
+	int id = fontID(fontname, ptsize);
+	if (id >= 0)
+		return id;
+	unique_ptr<NativeFont> nativeFont;
+	id = fontID(fontname);
+	if (id < 0) {
+		nativeFont = util::make_unique<NativeFontImpl>(fname, fontname, ptsize);
+		_name2id.emplace(std::move(fontname), int(_fonts.size()));
+	}
+	else {
+		auto *nf = font_cast<NativeFont*>(getFontById(id));
+		nativeFont = unique_ptr<NativeFont>(nf->clone(ptsize, FontStyle(), Color::BLACK));
+	}
+	id = int(_fonts.size());
+	_fonts.push_back(std::move(nativeFont));
+	return id;
+}
+
+
 /** Enters a new virtual font frame.
  *  This method must be called before processing a VF character.
  *  @param[in] vf virtual font */
@@ -322,6 +385,20 @@ void FontManager::assignVFChar (int c, vector<uint8_t> &&dvi) {
 }
 
 
+void FontManager::addUsedChar (const Font &font, int c) {
+	_usedChars[font.uniqueFont()].insert(c);
+	if (!SVGTree::USE_FONTS)
+		_usedChars[&font].insert(c);
+	_usedFonts.insert(&font);
+}
+
+
+void FontManager::resetUsedChars () {
+	_usedChars.clear();
+	_usedFonts.clear();
+}
+
+
 ostream& FontManager::write (ostream &os, Font *font, int level) {
 #if 0
 	if (font) {
@@ -330,7 +407,7 @@ ostream& FontManager::write (ostream &os, Font *font, int level) {
 			if (_fonts[i] == font)
 				id = i;
 
-		VirtualFont *vf = dynamic_cast<VirtualFont*>(font);
+		VirtualFont *vf = font_cast<VirtualFont*>(font);
 		for (int j=0; j < level+1; j++)
 			os << "  ";
 		os << "id " << id
