@@ -1,3 +1,7 @@
+#if defined(MIKTEX_WINDOWS)
+#include <miktex/Util/CharBuffer>
+#define UW_(x) MiKTeX::Util::CharBuffer<wchar_t>(x).GetData()
+#endif
 #include "jsfile.h"
 
 #include "settings.h"
@@ -8,9 +12,43 @@ using namespace settings;
 
 namespace camp {
 
+#ifndef HAVE_LIBGLM
+size_t materialIndex=0;
+#endif
+
+jsfile::jsfile() : finished(false), fileName("")
+{
+
+}
+
+jsfile::jsfile(string name) : finished(false), fileName(name)
+{
+  open(name);
+}
+
+jsfile::~jsfile()
+{
+  if (!finished)
+    {
+      finish(fileName);
+    }
+}
+
+void jsfile::close()
+{
+  if (!finished)
+    {
+      finish(fileName);
+    }
+}
+
 void jsfile::copy(string name, bool header)
 {
+#if defined(MIKTEX_WINDOWS)
+  std::ifstream fin(UW_(locateFile(name)));
+#else
   std::ifstream fin(locateFile(name).c_str());
+#endif
   string s;
   if(header) getline(fin,s);
   while(getline(fin,s))
@@ -68,12 +106,12 @@ void jsfile::svgtohtml(string prefix)
   out << "<body>" << newl << newl;
   copy(locateFile(auxname(prefix,"svg")),true);
   footer(name);
+  finished=true;
 }
 
-#ifdef HAVE_LIBGLM
-
 void jsfile::comment(string name)
 {
+#ifdef HAVE_LIBGLM
   out << "<!-- Use the following line to embed this file within another web page:" << newl
       << newl
       << "<iframe src=\"" << name
@@ -82,6 +120,7 @@ void jsfile::comment(string name)
       << "\" frameborder=\"0\"></iframe>" << newl
       << newl
       << "-->" << newl << newl;
+#endif
 }
 
 void jsfile::open(string name)
@@ -90,35 +129,51 @@ void jsfile::open(string name)
   comment(name);
   meta(name,false);
 
+#ifdef HAVE_LIBGLM
   out.precision(getSetting<Int>("digits"));
 
+  bool ibl=getSetting<bool>("ibl");
+  bool webgl2=ibl || getSetting<bool>("webgl2");
+  if(ibl)
+    out << "<script src=\"https://vectorgraphics.gitlab.io/asymptote/ibl/tinyexr.js\">"
+        << newl << "</script>" << newl;
+
   if(getSetting<bool>("offline")) {
     out << "<script>" << newl;
     copy(locateFile(AsyGL));
     out << newl << "</script>" << newl;
-  } else {
+  } else
     out << "<script" << newl << "src=\""
         << getSetting<string>("asygl") << "\">" << newl << "</script>" << newl;
-  }
+
   out << newl << "<script>" << newl;
   out << newl
       << "canvasWidth=" << gl::fullWidth << ";" << newl
-      << "canvasHeight=" << gl::fullHeight << ";" << newl
-      << "absolute=" << std::boolalpha << getSetting<bool>("absolute") << ";"
-      << newl << newl
-      <<  "b=[" << gl::xmin << "," << gl::ymin << "," << gl::zmin << "];"
+      << "canvasHeight=" << gl::fullHeight << ";" << newl << newl
+      << "webgl2=" << std::boolalpha << webgl2 << ";"
       << newl
-      <<  "B=[" << gl::xmax << "," << gl::ymax << "," << gl::zmax << "];"
+      << "ibl=" << std::boolalpha << ibl << ";"
+      << newl
+      << "absolute=" << std::boolalpha << getSetting<bool>("absolute") << ";"
+      << newl;
+  if(ibl) {
+    out << "imageURL=\"" << getSetting<string>("imageURL")+"/\";" << newl;
+    out << "image=\"" << getSetting<string>("image") << "\";" << newl << newl;
+  }
+  out << newl
+      <<  "minBound=[" << gl::Xmin << "," << gl::Ymin << "," << gl::Zmin << "];"
+      << newl
+      <<  "maxBound=[" << gl::Xmax << "," << gl::Ymax << "," << gl::Zmax << "];"
       << newl
       << "orthographic=" << gl::orthographic << ";"
       << newl
-      << "angle=" << gl::Angle << ";"
+      << "angleOfView=" << gl::Angle << ";"
       << newl
-      << "Zoom0=" << gl::Zoom0 << ";" << newl
-      << "viewportmargin=" << gl::Margin << ";" << newl;
-  if(gl::Shift != pair(0.0,0.0))
-    out << "viewportshift=" << gl::Shift*gl::Zoom0 << ";" << newl;
-  out << "zoomFactor=" << getSetting<double>("zoomfactor") << ";" << newl
+      << "initialZoom=" << gl::Zoom0 << ";" << newl;
+    if(gl::Shift != pair(0.0,0.0))
+      out << "viewportShift=" << gl::Shift*gl::Zoom0 << ";" << newl;
+    out << "viewportMargin=" << gl::Margin << ";" << newl << newl
+      << "zoomFactor=" << getSetting<double>("zoomfactor") << ";" << newl
       << "zoomPinchFactor=" << getSetting<double>("zoomPinchFactor") << ";"
       << newl
       << "zoomPinchCap=" << getSetting<double>("zoomPinchCap") << ";" << newl
@@ -133,31 +188,33 @@ void jsfile::open(string name)
   for(size_t i=0; i < gl::nlights; ++i) {
     size_t i4=4*i;
     out << "new Light(" << newl
-        << "direction=" << gl::Lights[i] << "," << newl
-        << "color=[" << gl::Diffuse[i4] << "," << gl::Diffuse[i4+1] << ","
+        << gl::Lights[i] << "," << newl
+        << "[" << gl::Diffuse[i4] << "," << gl::Diffuse[i4+1] << ","
         << gl::Diffuse[i4+2] << "])," << newl;
   }
   out << "];" << newl << newl;
   out << "Background=[" << gl::Background[0] << "," << gl::Background[1] << ","
       << gl::Background[2] << "," << gl::Background[3] << "];"
-      << newl;
-
-  size_t nmaterials=material.size();
-  out << "Materials=[";
-  for(size_t i=0; i < nmaterials; ++i)
-    out << "new Material(" << newl
-        << material[i]
-        << ")," << newl;
+      << newl << newl;
+  out << "Transform=[" << gl::T[0];
+  for(int i=1; i < 16; ++i)
+    out << "," << newl << gl::T[i];
   out << "];" << newl << newl;
+
+  camp::clearCenters();
+  camp::clearMaterials();
+#endif
 }
 
 void jsfile::finish(string name)
 {
-  size_t ncenters=drawElement::center.size();
+#ifdef HAVE_LIBGLM
+  finished=true;
+  size_t ncenters=drawElement::centers.size();
   if(ncenters > 0) {
     out << "Centers=[";
     for(size_t i=0; i < ncenters; ++i)
-      out << newl << drawElement::center[i] << ",";
+      out << newl << drawElement::centers[i] << ",";
     out << newl << "];" << newl;
   }
   out << "</script>"
@@ -165,9 +222,10 @@ void jsfile::finish(string name)
       << newl << newl << "<body style=\"overflow: hidden;\" onload=\"webGLStart();\">"
       << newl << "<canvas id=\"Asymptote\" width=\""
       << gl::fullWidth << "\" height=\"" <<  gl::fullHeight
-      << "\" style=\"border: none;\">"
+      << "\" style=\"border: none; cursor: pointer;\">"
       << newl << "</canvas>";
   footer(name);
+#endif
 }
 
 void jsfile::addColor(const prc::RGBAColour& c)
@@ -181,22 +239,17 @@ void jsfile::addIndices(const uint32_t *I)
   out << "[" << I[0] << "," << I[1] << "," << I[2] << "]";
 }
 
-bool distinct(const uint32_t *I, const uint32_t *J)
+void jsfile::addRawPatch(triple const* controls, size_t n,
+                         const triple& Min, const triple& Max,
+                         const prc::RGBAColour *c, size_t nc)
 {
-  return I[0] != J[0] || I[1] != J[1] || I[2] != J[2];
-}
-
-void jsfile::addPatch(triple const* controls, size_t n,
-                      const triple& Min, const triple& Max,
-                      const prc::RGBAColour *c, size_t nc)
-{
-  out << "P.push(new BezierPatch([" << newl;
+  out << "patch([" << newl;
   size_t last=n-1;
   for(size_t i=0; i < last; ++i)
     out << controls[i] << "," << newl;
   out << controls[last] << newl << "],"
       << drawElement::centerIndex << "," << materialIndex << ","
-      << Min << "," << Max;
+      << newl << Min << "," << newl << Max;
   if(c) {
     out << ",[" << newl;
     for(size_t i=0; i < nc; ++i) {
@@ -205,46 +258,49 @@ void jsfile::addPatch(triple const* controls, size_t n,
     }
     out << "]";
   }
-  out << "));" << newl << newl;
+  out << ");" << newl << newl;
 }
 
 void jsfile::addCurve(const triple& z0, const triple& c0,
                       const triple& c1, const triple& z1,
                       const triple& Min, const triple& Max)
 {
-  out << "P.push(new BezierCurve([" << newl;
+  out << "curve([" << newl;
   out << z0 << "," << newl
       << c0 << "," << newl
       << c1 << "," << newl
       << z1 << newl << "],"
       << drawElement::centerIndex << "," << materialIndex << ","
-      << Min << "," << Max << "));" << newl << newl;
+      << newl << Min << "," << newl << Max << ");" << newl << newl;
 }
 
 void jsfile::addCurve(const triple& z0, const triple& z1,
                       const triple& Min, const triple& Max)
 {
-  out << "P.push(new BezierCurve([" << newl;
+  out << "curve([" << newl;
   out << z0 << "," << newl
       << z1 << newl << "],"
       << drawElement::centerIndex << "," << materialIndex << ","
-      << Min << "," << Max << "));" << newl << newl;
+      << newl << Min << "," << newl << Max << ");" << newl << newl;
 }
 
 void jsfile::addPixel(const triple& z0, double width,
                       const triple& Min, const triple& Max)
 {
-  out << "P.push(new Pixel(" << newl;
+  out << "pixel(" << newl;
   out << z0 << "," << width << "," << newl
-      << materialIndex << "," << Min << "," << Max << "));" << newl << newl;
+      << materialIndex << "," << newl << Min << "," << newl << Max << ");"
+      << newl << newl;
 }
 
-void jsfile::addMaterial(size_t index)
+#ifdef HAVE_LIBGLM
+void jsfile::addMaterial(Material const& material)
 {
-  out << "Materials.push(new Material(" << newl
-      << material[index]
-      << "));" << newl << newl;
+  out << "material(" << newl
+      << material
+      << ");" << newl << newl;
 }
+#endif
 
 void jsfile::addTriangles(size_t nP, const triple* P, size_t nN,
                           const triple* N, size_t nC, const prc::RGBAColour* C,
@@ -281,18 +337,24 @@ void jsfile::addTriangles(size_t nP, const triple* P, size_t nN,
     }
     out << "]);" << newl;
   }
-  out << "P.push(new Triangles("
-      << materialIndex << "," << newl
-      << Min << "," << Max << "));" << newl << newl;
+  out << "triangles("
+      << drawElement::centerIndex << "," << materialIndex << "," << newl
+      << newl << Min << "," << newl << Max << ");" << newl << newl;
 }
 
-void jsfile::addSphere(const triple& center, double radius, bool half,
-                       const double& polar, const double& azimuth)
+void jsfile::addSphere(const triple& center, double radius)
 {
   out << "sphere(" << center << "," << radius << ","
-      << drawElement::centerIndex << "," << materialIndex;
-  if(half)
-    out << "," << newl << "[" << polar << "," << azimuth << "]";
+      << drawElement::centerIndex << "," << materialIndex
+      << ");" << newl << newl;
+}
+
+void jsfile::addHemisphere(const triple& center, double radius,
+                           const double& polar, const double& azimuth)
+{
+  out << "sphere(" << center << "," << radius << ","
+      << drawElement::centerIndex << "," << materialIndex
+      << "," << newl << "[" << polar << "," << azimuth << "]";
   out << ");" << newl << newl;
 }
 
@@ -328,8 +390,36 @@ void jsfile::addTube(const triple *g, double width,
       << g[3] << newl << "],"
       << width << ","
       << drawElement::centerIndex << "," << materialIndex << ","
-      << Min << "," << Max << "," << core <<");" << newl << newl;
+      << newl << Min << "," << newl << Max << "," << core <<");"
+      << newl << newl;
+}
+
+void jsfile::addPatch(triple const* controls,
+                      triple const& Min, triple const& Max,
+                      prc::RGBAColour const* c)
+{
+  addRawPatch(controls,16,Min,Max,c,4);
+}
+
+void jsfile::addStraightPatch(triple const* controls,
+                              triple const& Min, triple const& Max,
+                              prc::RGBAColour const* c)
+{
+  addRawPatch(controls,4,Min,Max,c,4);
+}
+
+void jsfile::addBezierTriangle(triple const* controls,
+                               triple const& Min, triple const& Max,
+                               prc::RGBAColour const* c)
+{
+  addRawPatch(controls,10,Min,Max,c,3);
+}
+
+void jsfile::addStraightBezierTriangle(triple const* controls,
+                                       triple const& Min, triple const& Max,
+                                       prc::RGBAColour const* c)
+{
+  addRawPatch(controls,3,Min,Max,c,3);
 }
-#endif
 
 }
