@@ -3,7 +3,7 @@
  * @author Christian Schenk
  * @brief MiKTeX MakeFMT
  *
- * @copyright Copyright © 1998-2022 Christian Schenk
+ * @copyright Copyright © 1998-2023 Christian Schenk
  *
  * This file is part of the MiKTeX Make Utility Collection.
  *
@@ -50,9 +50,7 @@ enum class Engine
     XeTeX,
     epTeX,
     eupTeX,
-    pTeX,
     pdfTeX,
-    upTeX,
 };
 
 class PdfConfigValues :
@@ -112,14 +110,10 @@ public:
             return "hitex";
         case Engine::pdfTeX:
             return "pdftex";
-        case Engine::pTeX:
-            return "ptex";
         case Engine::epTeX:
             return "eptex";
         case Engine::eupTeX:
             return "euptex";
-        case Engine::upTeX:
-            return "uptex";
         case Engine::XeTeX:
             return "xetex";
         case Engine::LuaHBTeX:
@@ -155,10 +149,6 @@ private:
         {
             this->engine = Engine::pdfTeX;
         }
-        else if (Utils::EqualsIgnoreCase(engine, "ptex"))
-        {
-            this->engine = Engine::pTeX;
-        }
         else if (Utils::EqualsIgnoreCase(engine, "eptex"))
         {
             this->engine = Engine::epTeX;
@@ -167,10 +157,6 @@ private:
         {
             this->engine = Engine::eupTeX;
         }
-        else if (Utils::EqualsIgnoreCase(engine, "uptex"))
-        {
-            this->engine = Engine::upTeX;
-        }
         else if (Utils::EqualsIgnoreCase(engine, "xetex"))
         {
             this->engine = Engine::XeTeX;
@@ -202,14 +188,10 @@ private:
             return MIKTEX_HITEX_EXE;
         case Engine::pdfTeX:
             return MIKTEX_PDFTEX_EXE;
-        case Engine::pTeX:
-            return MIKTEX_PTEX_EXE;
         case Engine::epTeX:
             return MIKTEX_EPTEX_EXE;
         case Engine::eupTeX:
             return MIKTEX_EUPTEX_EXE;
-        case Engine::upTeX:
-            return MIKTEX_UPTEX_EXE;
         case Engine::XeTeX:
             return MIKTEX_XETEX_EXE;
         case Engine::LuaHBTeX:
