@@ -820,6 +820,12 @@ static int getpdfomitinfodict(lua_State * L)
     return 1 ;
 }
 
+static int getpdfomitmediabox(lua_State * L)
+{
+    lua_pushinteger(L, (pdf_omit_mediabox));
+    return 1 ;
+}
+
 static int setpdfgentounicode(lua_State * L)
 {
     if (lua_type(L, 1) == LUA_TNUMBER) {
@@ -852,6 +858,14 @@ static int setpdfomitinfodict(lua_State * L)
     return 0 ;
 }
 
+static int setpdfomitmediabox(lua_State * L)
+{
+    if (lua_type(L, 1) == LUA_TNUMBER) {
+        set_pdf_omit_mediabox(lua_tointeger(L, 1));
+    }
+    return 0 ;
+}
+
 /* for tracing purposes when no pages are flushed */
 
 static int setforcefile(lua_State * L)
@@ -1034,7 +1048,7 @@ static int setpdforigin(lua_State * L)
     if (lua_type(L, 1) == LUA_TNUMBER) {
         h = (int) lua_roundnumber(L, 1);
         if (lua_type(L, 2) == LUA_TNUMBER) {
-            v = (int) lua_roundnumber(L, 1);
+            v = (int) lua_roundnumber(L, 2);
         } else {
             v = h;
         }
@@ -1365,12 +1379,14 @@ static const struct luaL_Reg pdflib[] = {
     { "getomitcidset", getpdfomitcidset },
     { "getomitcharset", getpdfomitcharset },
     { "getomitinfo", getpdfomitinfodict },
+    { "getomitmediabox", getpdfomitmediabox },
     { "setinclusionerrorlevel", setpdfinclusionerrorlevel },
     { "setignoreunknownimages", setpdfignoreunknownimages },
     { "setgentounicode", setpdfgentounicode },
     { "setomitcidset", setpdfomitcidset },
     { "setomitcharset", setpdfomitcharset },
     { "setomitinfo", setpdfomitinfodict },
+    { "setomitmediabox", setpdfomitmediabox },
     { "setforcefile", setforcefile },
     { "mapfile", l_mapfile },
     { "mapline", l_mapline },
